@@ -7,33 +7,29 @@ package DBIx::DataModel::Schema;
 
 use warnings;
 use strict;
-use DBIx::DataModel::Meta::Utils qw/does/;
+use Carp;
 use DBIx::DataModel::Source::Table;
 
-use Scalar::Util     qw/blessed/;
+use Scalar::Util     qw/blessed reftype/;
 use Module::Load     qw/load/;
-use Params::Validate qw/validate_with SCALAR ARRAYREF CODEREF UNDEF 
-                                      OBJECT BOOLEAN/;
+use Params::Validate qw/validate SCALAR ARRAYREF CODEREF UNDEF 
+                                 OBJECT BOOLEAN/;
 use Acme::Damn       qw/damn/;
-use Carp::Clan       qw[^(DBIx::DataModel::|SQL::Abstract)];
-
-use SQL::Abstract::More 1.33;
-use Try::Tiny;
+use SQL::Abstract::More;
 
 use namespace::clean;
 
+{no strict 'refs'; *CARP_NOT = \@DBIx::DataModel::CARP_NOT;}
 
 my $spec = {
-  dbh                   => {type => OBJECT|ARRAYREF, optional => 1},
-  debug                 => {type => OBJECT|SCALAR,   optional => 1},
-  sql_abstract          => {type => OBJECT,
-                            isa  => 'SQL::Abstract::More',
-                            optional => 1},
-  dbi_prepare_method    => {type => SCALAR,  default  => 'prepare'},
-  placeholder_prefix    => {type => SCALAR,  default  => '?:'},
-  select_implicitly_for => {type => SCALAR,  default  => ''},
-  autolimit_firstrow    => {type => BOOLEAN, optional => 1},
-  db_schema             => {type => SCALAR,  optional => 1},
+  dbh                          => {type => OBJECT|ARRAYREF, optional => 1},
+  debug                        => {type => OBJECT|SCALAR,   optional => 1},
+  sql_abstract                 => {type => OBJECT,
+                                   isa  => 'SQL::Abstract::More',
+                                   optional => 1},
+  dbi_prepare_method           => {type => SCALAR, default => 'prepare'},
+  placeholder_prefix           => {type => SCALAR, default => '?:'},
+  select_implicitly_for        => {type => SCALAR, default => ''},
 };
 
 
@@ -45,11 +41,7 @@ sub new {
     or croak "$class is already used in single-schema mode, can't call new()";
 
   # validate params
-  my %params = validate_with(
-    params      => \@_,
-    spec        => $spec,
-    allow_extra => 0,
-   );
+  my %params = validate(@_, $spec);
 
   # instantiate and call 'setter' methods for %params
   my $self = bless {}, $class;
@@ -112,7 +104,7 @@ sub dbh {
 
     # also support syntax ->dbh([$dbh, %dbh_options])
     ($dbh, %dbh_options) = @$dbh 
-      if does($dbh, 'ARRAY') && ! keys %dbh_options;
+      if $dbh && ref $dbh eq 'ARRAY' && ! keys %dbh_options;
 
     # forbid change of dbh while doing a transaction
     not $self->{dbh} or $self->{dbh}[0]{AutoCommit}
@@ -127,14 +119,6 @@ sub dbh {
       $dbh->{RaiseError} 
         or croak "arg to dbh(..) must have RaiseError=1";
 
-      # default values for $dbh_options{returning_through}
-      if (not exists $dbh_options{returning_through}) {
-        for ($dbh->{Driver}{Name}) {
-          /^Oracle/ and do {$dbh_options{returning_through} = 'INOUT'; last};
-          /^Pg/     and do {$dbh_options{returning_through} = 'FETCH'; last};
-        }
-      }
-
       # store the dbh
       $self->{dbh} = [$dbh, %dbh_options];
     }
@@ -152,8 +136,7 @@ sub dbh {
 
 # some rw setters/getters
 my @accessors = qw/debug select_implicitly_for dbi_prepare_method 
-                   sql_abstract placeholder_prefix autolimit_firstrow
-                   db_schema/;
+                   sql_abstract placeholder_prefix/;
 foreach my $accessor (@accessors) {
   no strict 'refs';
   *$accessor = sub {
@@ -168,17 +151,10 @@ foreach my $accessor (@accessors) {
 }
 
 
-sub with_db_schema {
-  my ($self, $db_schema) = @_;
-  ref $self or $self = $self->singleton;
-
-  # return a shallow copy of $self with db_schema set to the given arg
-  return bless { %$self, db_schema => $db_schema}, ref $self;
-}
 
 
 my @default_state_components = qw/dbh debug select_implicitly_for
-                                  dbi_prepare_method db_schema/;
+                                  dbi_prepare_method /;
 
 sub localize_state {
   my ($self, @components) = @_; 
@@ -194,24 +170,10 @@ sub localize_state {
 
 
 
-
-sub do_after_commit {
-  my ($self, $coderef) = @_;
-  ref $self or $self = $self->singleton;
-
-  $self->{transaction_dbhs}
-    or croak "do_after_commit() called outside of a transaction";
-  push @{$self->{after_commit_callbacks}}, $coderef;
-}
-
-
 sub do_transaction { 
   my ($self, $coderef, @new_dbh) = @_; 
   ref $self or $self = $self->singleton;
 
-  does($coderef, 'CODE')
-    or croak 'first arg to $schema->do_transaction(...) should be a coderef';
-
   my $transaction_dbhs = $self->{transaction_dbhs} ||= [];
 
   # localize the dbh and its options, if so requested. 
@@ -222,7 +184,7 @@ sub do_transaction {
     if @new_dbh; # postfix "if" because $local_state must not be in a block
 
   # check that we have a dbh
-  my $dbh = $self->dbh
+  my $dbh = $self->{dbh}[0]
     or croak "no database handle for transaction";
 
   # how to call and how to return will depend on context
@@ -254,59 +216,29 @@ sub do_transaction {
     $begin_work_and_exec->();
   }
   else { # else try to execute and commit in an eval block
+    eval {
+      # check AutoCommit state
+      $dbh->{AutoCommit}
+        or croak "dbh was not in Autocommit mode before initial transaction";
 
-    # support for DBIx::RetryOverDisconnects: decide how many retries
-    my $n_retries = 1;
-    if ($dbh->isa('DBIx::RetryOverDisconnects::db')) {
-      $n_retries = $dbh->{DBIx::RetryOverDisconnects::PRIV()}{txn_retries};
-    }
+      # execute the transaction
+      $begin_work_and_exec->();
 
-    # try to do the transaction, maybe several times in cas of disconnection
-  RETRY:
-    for my $retry (1 .. $n_retries) {
-      no warnings 'exiting'; # because "last/next" are in Try::Tiny subroutines
-      try {
-        # check AutoCommit state
-        $dbh->{AutoCommit}
-          or croak "dbh was not in Autocommit mode before initial transaction";
+      # commit all dbhs and then reset the list of dbhs
+      $_->commit foreach @$transaction_dbhs;
+      delete $self->{transaction_dbhs};
+    };
 
-        # execute the transaction
-        $begin_work_and_exec->();
-
-        # commit all dbhs and then reset the list of dbhs
-        $_->commit foreach @$transaction_dbhs;
-        delete $self->{transaction_dbhs};
-
-        # execute the after_commit callbacks
-        my $callbacks = delete $self->{after_commit_callbacks} || [];
-        $_->() foreach @$callbacks;
-
-        last RETRY; # transaction successful, get out of the loop
-      }
-      catch {
-        my $err = $_;
-
-        # if this was a disconnection ..
-        if ($dbh->isa('DBIx::RetryOverDisconnects::db') 
-              # $dbh->can() is broken on DBI handles, so use ->isa() instead
-              && $dbh->is_trans_disconnect) {
-          $transaction_dbhs = [];
-          next RETRY if $retry < $n_retries;   # .. try again
-          $self->exc_conn_trans_fatal->throw;  # .. or no hope (and no rollback)
-        }
-
-        # otherwise, for regular SQL errors, try to rollback and then throw
-        my @rollback_errs;
-        foreach my $dbh (reverse @$transaction_dbhs) {
-          try   {$dbh->rollback}
-            catch {push @rollback_errs, $_};
-        }
-        delete $self->{transaction_dbhs};
-        delete $self->{after_commit_callbacks};
-        DBIx::DataModel::Schema::_Exception->throw($err, @rollback_errs);
-      };
+    # if any error, rollback
+    my $err = $@;
+    if ($err) {              # the transaction failed
+      my @rollback_errs = grep {$_} map {eval{$_->rollback}; $@} 
+                                        reverse @$transaction_dbhs;
+      delete $self->{transaction_dbhs};
+      DBIx::DataModel::Schema::_Exception->throw($err, @rollback_errs);
     }
   }
+
   return $in_context->{return}->();
 }
 
@@ -325,8 +257,7 @@ sub unbless {
 #                   local method     metadm method
 #                   ============     =============
 my %accessor_map = (table         => 'table',
-                    join          => 'define_join',
-                    db_table      => 'db_table');
+                    join          => 'define_join');
 while (my ($local, $remote) = each %accessor_map) {
   no strict 'refs';
   *$local = sub {
@@ -334,23 +265,12 @@ while (my ($local, $remote) = each %accessor_map) {
     ref $self or $self = $self->singleton;
 
     my $meta_source = $self->metadm->$remote(@_) or return;
-    my $obj = bless {__schema => $self}, $meta_source->class;
-    return $obj;
+    my $cs_class = $self->metadm->connected_source_class;
+    load $cs_class;
+    return $cs_class->new($meta_source, $self);
   }
 }
 
-# while (my ($local, $remote) = each %accessor_map) {
-#   no strict 'refs';
-#   *$local = sub {
-#     my $self = shift;
-#     ref $self or $self = $self->singleton;
-
-#     my $meta_source = $self->metadm->$remote(@_) or return;
-#     my $cs_class = $self->metadm->connected_source_class;
-#     load $cs_class;
-#     return $cs_class->new($meta_source, $self);
-#   }
-# }
 
 #----------------------------------------------------------------------
 # UTILITY FUNCTIONS (PRIVATE)
@@ -386,8 +306,7 @@ sub _debug { # internal method to send debug messages
 # PRIVATE CLASS FOR LOCALIZING STATE (see L</localizeState> method
 #----------------------------------------------------------------------
 
-package
-  DBIx::DataModel::Schema::_State;
+package DBIx::DataModel::Schema::_State;
 
 sub new {
   my ($class, $schema, $state) = @_;
@@ -417,8 +336,7 @@ sub DESTROY { # called when the guard goes out of scope
 # PRIVATE CLASS FOR TRANSACTION EXCEPTIONS
 #----------------------------------------------------------------------
 
-package
-  DBIx::DataModel::Schema::_Exception;
+package DBIx::DataModel::Schema::_Exception;
 use strict;
 use warnings;
 
@@ -452,8 +370,6 @@ sub rollback_errors {
 
 __END__
 
-=encoding ISO8859-1
-
 =head1 NAME
 
 DBIx::DataModel::Schema - Factory for DBIx::DataModel Schemas
@@ -466,66 +382,56 @@ This is the parent class for all schema classes created through
 
 =head1 CONSTRUCTOR
 
-See L<DBIx::DataModel::Doc::Reference/Schema>
-
+[TODO]
 
 =head1 METHODS
 
 Methods are documented in 
 L<DBIx::DataModel::Doc::Reference|DBIx::DataModel::Doc::Reference>.
-
-=head2 Delegated methods
-
-Methods delegated to L<DBIx::DataModel::Meta::Schema> :
+This module implements
 
 =over
 
+=item L<Schema|DBIx::DataModel::Doc::Reference/Schema>
+
 =item L<Table|DBIx::DataModel::Doc::Reference/Table>
 
 =item L<View|DBIx::DataModel::Doc::Reference/View>
 
 =item L<Association|DBIx::DataModel::Doc::Reference/Association>
 
-=item L<Composition|DBIx::DataModel::Doc::Reference/Composition>
-
-=item L<Type|DBIx::DataModel::Doc::Reference/Type>
-
-=item L<table|DBIx::DataModel::Doc::Reference/table>
-
 =item L<join|DBIx::DataModel::Doc::Reference/join>
 
-=back
-
-
-
-=head2 Implemented methods
-
-Methods implemented in this module :
-
-=over
-
-=item L<singleton|DBIx::DataModel::Doc::Reference/singleton>
+=item L<ColumnType|DBIx::DataModel::Doc::Reference/ColumnType>
 
 =item L<dbh|DBIx::DataModel::Doc::Reference/dbh>
 
 =item L<debug|DBIx::DataModel::Doc::Reference/debug>
 
-=item L<select_implicitly_for|DBIx::DataModel::Doc::Reference/select_implicitly_for>
+=item L<noUpdateColumns|DBIx::DataModel::Doc::Reference/noUpdateColumns>
 
-=item L<dbi_prepare_method|DBIx::DataModel::Doc::Reference/dbi_prepare_method>
+=item L<autoUpdateColumns|DBIx::DataModel::Doc::Reference/autoUpdateColumns>
 
-=item L<sql_abstract|DBIx::DataModel::Doc::Reference/sql_abstract>
+=item L<selectImplicitlyFor|DBIx::DataModel::Doc::Reference/selectImplicitlyFor>
 
-=item L<placeholder_prefix|DBIx::DataModel::Doc::Reference/placeholder_prefix>
+=item L<dbiPrepareMethod|DBIx::DataModel::Doc::Reference/dbiPrepareMethod>
 
-=item L<autolimit_firstrow|DBIx::DataModel::Doc::Reference/autolimit_firstrow>
+=item L<tables|DBIx::DataModel::Doc::Reference/tables>
 
-=item L<localize_state|DBIx::DataModel::Doc::Reference/localize_state>
+=item L<table|DBIx::DataModel::Doc::Reference/table>
 
-=item L<do_transaction|DBIx::DataModel::Doc::Reference/do_transaction>
+=item L<localizeState|DBIx::DataModel::Doc::Reference/localizeState>
+
+=item L<statementClass|DBIx::DataModel::Doc::Reference/statementClass>
+
+=item L<doTransaction|DBIx::DataModel::Doc::Reference/doTransaction>
 
 =item L<unbless|DBIx::DataModel::Doc::Reference/unbless>
 
+=item L<_createPackage|DBIx::DataModel::Doc::Reference/_createPackage>
+
+=item L<_defineMethod|DBIx::DataModel::Doc::Reference/_defineMethod>
+
 =back
 
 =head1 PRIVATE SUBCLASSES
@@ -539,7 +445,9 @@ A private class for localizing state (using a DESTROY method).
 =head2 _Exception
 
 A private class for exceptions during transactions
-(see  L<do_transaction|DBIx::DataModel::Doc::Reference/do_transaction>).
+(see  L<doTransaction|DBIx::DataModel::Doc::Reference/doTransaction>).
+
+
 
 =head1 AUTHOR
 
@@ -547,8 +455,11 @@ Laurent Dami, E<lt>laurent.dami AT etat  ge  chE<gt>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2006-2012 Laurent Dami.
+Copyright 2006, 2008 Laurent Dami.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
+
+
+
